﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace TriTech.VisiCAD.App.WPF
{
    [TemplateVisualState(GroupName = "EditStates", Name = "EditMode")]
    [TemplateVisualState(GroupName = "EditStates", Name = "ViewMode")]
    public class EditableContentControl : ContentControl
    {
        private TextBox textBox;
        private DateTime previosLeftClickTime = DateTime.Now;
        private Point previosLeftClickPoint;
        private TimeSpan doubleClickSpan = TimeSpan.FromSeconds(0.4);

        public EditableContentControl()
        {
            DefaultStyleKey = typeof(EditableContentControl);
        }

        protected override void OnContentChanged(object oldContent, object newContent)
        {
            base.OnContentChanged(oldContent, newContent);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            this.textBox = this.GetTemplateChild("TextBox") as TextBox;

			this.textBox.GotFocus += textBox_GotFocus;

            this.textBox.LostFocus += new RoutedEventHandler(textBox_LostFocus);
        }

		void textBox_Loaded(object sender, RoutedEventArgs e)
		{
		}

		private void textBox_GotFocus(object sender, RoutedEventArgs e)
		{
			this.textBox.SelectAll();
		}

        void textBox_LostFocus(object sender, RoutedEventArgs e)
        {
            this.IsInEditMode = false;
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);

            var currentTime = DateTime.Now;
            var currentPoint = e.GetPosition(this);
            var durationBetweenClicks = currentTime - previosLeftClickTime;

            if (currentPoint == previosLeftClickPoint && durationBetweenClicks < this.doubleClickSpan)
            {
                e.Handled = true;
                this.IsInEditMode = !this.IsInEditMode;
            }

            this.previosLeftClickTime = DateTime.Now;
            this.previosLeftClickPoint = e.GetPosition(this);
        }

        public bool IsInEditMode
        {
            get { return (bool)GetValue(IsInEditModeProperty); }
            set { SetValue(IsInEditModeProperty, value); }
        }

        public static readonly DependencyProperty IsInEditModeProperty =
            DependencyProperty.Register("IsInEditMode", typeof(bool), typeof(EditableContentControl), new PropertyMetadata(OnIsInEditModeChanged));

        private static void OnIsInEditModeChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var editableContentControl = sender as EditableContentControl;
            var newValue = (bool)e.NewValue;

            if (!newValue)
            {
                editableContentControl.Content = editableContentControl.textBox.Text;
            }

            editableContentControl.ChangeVisualStates();
        }

        public void ChangeVisualStates()
        {
            if (this.IsInEditMode)
            {
                VisualStateManager.GoToState(this, "EditMode", true);
            }
            else
            {
                VisualStateManager.GoToState(this, "ViewMode", true);
            }
        }
    }
}
